@@ -71,6 +71,8 @@ typedef struct
   guint setup_idle_id;
   gboolean dispose_has_run;
   GHashTable *status_icons;
+  /* List of owned GCancellables for each pending avatar load operation */
+  GList *avatar_cancellables;
 } EmpathyIndividualStorePriv;
 
 typedef struct
@@ -552,13 +554,16 @@ individual_store_contact_active_cb (ShowActiveData *data)
   return FALSE;
 }
 
+typedef struct {
+  EmpathyIndividualStore *store; /* weak */
+  GCancellable *cancellable; /* owned */
+} LoadAvatarData;
+
 static void
-individual_avatar_pixbuf_received_cb (GObject *object,
+individual_avatar_pixbuf_received_cb (FolksIndividual *individual,
     GAsyncResult *result,
-    gpointer user_data)
+    LoadAvatarData *data)
 {
-  FolksIndividual *individual = FOLKS_INDIVIDUAL (object);
-  EmpathyIndividualStore *self = user_data;
   GError *error = NULL;
   GdkPixbuf *pixbuf;
 
@@ -572,18 +577,32 @@ individual_avatar_pixbuf_received_cb (GObject *object,
           error->message);
       g_clear_error (&error);
     }
-  else
+  else if (data->store != NULL)
     {
       GList *iters, *l;
 
-      iters = individual_store_find_contact (self, individual);
+      iters = individual_store_find_contact (data->store, individual);
       for (l = iters; l; l = l->next)
         {
-          gtk_tree_store_set (GTK_TREE_STORE (self), l->data,
+          gtk_tree_store_set (GTK_TREE_STORE (data->store), l->data,
               EMPATHY_INDIVIDUAL_STORE_COL_PIXBUF_AVATAR, pixbuf,
               -1);
         }
     }
+
+  /* Free things */
+  if (data->store != NULL)
+    {
+      EmpathyIndividualStorePriv *priv = GET_PRIV (data->store);
+
+      g_object_remove_weak_pointer (G_OBJECT (data->store),
+          (gpointer *) &data->store);
+      priv->avatar_cancellables = g_list_remove (priv->avatar_cancellables,
+          data->cancellable);
+    }
+
+  g_object_unref (data->cancellable);
+  g_slice_free (LoadAvatarData, data);
 }
 
 static void
@@ -604,6 +623,7 @@ individual_store_contact_update (EmpathyIndividualStore *self,
   gboolean do_set_refresh = FALSE;
   gboolean show_avatar = FALSE;
   GdkPixbuf *pixbuf_status;
+  LoadAvatarData *load_avatar_data;
 
   priv = GET_PRIV (self);
 
@@ -678,8 +698,19 @@ individual_store_contact_update (EmpathyIndividualStore *self,
       show_avatar = TRUE;
     }
 
-  empathy_pixbuf_avatar_from_individual_scaled_async (individual,
-      individual_avatar_pixbuf_received_cb, 32, 32, self);
+  /* Load the avatar asynchronously */
+  load_avatar_data = g_slice_new (LoadAvatarData);
+  load_avatar_data->store = self;
+  g_object_add_weak_pointer (G_OBJECT (self),
+      (gpointer *) &load_avatar_data->store);
+  load_avatar_data->cancellable = g_cancellable_new ();
+
+  priv->avatar_cancellables = g_list_prepend (priv->avatar_cancellables,
+      load_avatar_data->cancellable);
+  empathy_pixbuf_avatar_from_individual_scaled_async (individual, 32, 32,
+      load_avatar_data->cancellable,
+      (GAsyncReadyCallback) individual_avatar_pixbuf_received_cb,
+      load_avatar_data);
 
   pixbuf_status =
       empathy_individual_store_get_individual_status_icon (self, individual);
@@ -771,7 +802,7 @@ static void
 individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
-  EmpathyContact *contact;
+  GList *personas, *l;
 
   g_signal_connect (individual, "notify::avatar",
       G_CALLBACK (individual_store_individual_updated_cb), self);
@@ -783,26 +814,67 @@ individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
       G_CALLBACK (individual_store_individual_updated_cb), self);
 
   /* FIXME: libfolks hasn't grown capabilities support yet, so we have to go
-   * through the EmpathyContact for them. */
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  g_object_set_data (G_OBJECT (contact), "individual", individual);
-  g_signal_connect (contact, "notify::capabilities",
-      G_CALLBACK (individual_store_contact_updated_cb), self);
-  g_object_unref (contact);
+   * through the EmpathyContacts for them. */
+  personas = folks_individual_get_personas (individual);
+  for (l = personas; l != NULL; l = l->next)
+    {
+      TpContact *tp_contact;
+      EmpathyContact *contact;
+
+      if (!TPF_IS_PERSONA (l->data))
+        continue;
+
+      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
+      contact = empathy_contact_dup_from_tp_contact (tp_contact);
+      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+
+      g_object_set_data (G_OBJECT (contact), "individual", individual);
+      g_signal_connect (contact, "notify::capabilities",
+          G_CALLBACK (individual_store_contact_updated_cb), self);
+
+      g_object_unref (contact);
+    }
 
   individual_store_add_individual (self, individual);
 }
 
+static void
+individual_store_disconnect_individual (EmpathyIndividualStore *self,
+    FolksIndividual *individual)
+{
+  GList *personas, *l;
+
+  g_signal_handlers_disconnect_by_func (individual,
+      G_CALLBACK (individual_store_individual_updated_cb), self);
+
+  /* FIXME: libfolks hasn't grown capabilities support yet, so we have to go
+   * through the EmpathyContacts for them. */
+  personas = folks_individual_get_personas (individual);
+  for (l = personas; l != NULL; l = l->next)
+    {
+      TpContact *tp_contact;
+      EmpathyContact *contact;
+
+      if (!TPF_IS_PERSONA (l->data))
+        continue;
+
+      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
+      contact = empathy_contact_dup_from_tp_contact (tp_contact);
+      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+
+      g_signal_handlers_disconnect_by_func (contact,
+          G_CALLBACK (individual_store_contact_updated_cb), self);
+
+      g_object_unref (contact);
+    }
+}
+
 static void
 individual_store_remove_individual_and_disconnect (
     EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
-  g_signal_handlers_disconnect_by_func (individual,
-      G_CALLBACK (individual_store_individual_updated_cb), self);
-  g_signal_handlers_disconnect_by_func (individual,
-      G_CALLBACK (individual_store_contact_updated_cb), self);
-
+  individual_store_disconnect_individual (self, individual);
   individual_store_remove_individual (self, individual);
 }
 
@@ -951,21 +1023,27 @@ static void
 individual_store_dispose (GObject *object)
 {
   EmpathyIndividualStorePriv *priv = GET_PRIV (object);
-  GList *contacts, *l;
+  GList *individuals, *l;
 
   if (priv->dispose_has_run)
     return;
   priv->dispose_has_run = TRUE;
 
-  contacts = empathy_individual_manager_get_members (priv->manager);
-  for (l = contacts; l; l = l->next)
+  /* Cancel any pending avatar load operations */
+  for (l = priv->avatar_cancellables; l != NULL; l = l->next)
     {
-      g_signal_handlers_disconnect_by_func (l->data,
-          G_CALLBACK (individual_store_individual_updated_cb), object);
-      g_signal_handlers_disconnect_by_func (l->data,
-          G_CALLBACK (individual_store_contact_updated_cb), object);
+      /* The cancellables are freed in individual_avatar_pixbuf_received_cb() */
+      g_cancellable_cancel (G_CANCELLABLE (l->data));
     }
-  g_list_free (contacts);
+  g_list_free (priv->avatar_cancellables);
+
+  individuals = empathy_individual_manager_get_members (priv->manager);
+  for (l = individuals; l; l = l->next)
+    {
+      individual_store_disconnect_individual (EMPATHY_INDIVIDUAL_STORE (object),
+          FOLKS_INDIVIDUAL (l->data));
+    }
+  g_list_free (individuals);
 
   g_signal_handlers_disconnect_by_func (priv->manager,
       G_CALLBACK (individual_store_member_renamed_cb), object);
@@ -1806,7 +1884,7 @@ individual_store_get_individual_status_icon_with_icon_name (
     {
       pixbuf_status =
           empathy_pixbuf_contact_status_icon_with_icon_name (contact,
-          status_icon_name, priv->show_protocols);
+          status_icon_name, show_protocols_here);
       if (pixbuf_status != NULL)
         {
           g_hash_table_insert (priv->status_icons,
