@@ -44,8 +44,7 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyIndividualManager)
 
-/* This class doesn't store or ref any of the individuals, since they're already
- * stored and referenced in the aggregator.
+/* This class only stores and refs Individuals who contain an EmpathyContact.
  *
  * This class merely forwards along signals from the aggregator and individuals
  * and wraps aggregator functions for other client code. */
@@ -53,6 +52,7 @@ typedef struct
 {
   FolksIndividualAggregator *aggregator;
   EmpathyContactManager *contact_manager;
+  GHashTable *individuals; /* Individual.id -> Individual */
 } EmpathyIndividualManagerPriv;
 
 enum
@@ -91,6 +91,76 @@ individual_notify_is_favourite_cb (FolksIndividual *individual,
       is_favourite);
 }
 
+static void
+add_individual (EmpathyIndividualManager *self, FolksIndividual *individual)
+{
+  EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
+
+  g_hash_table_insert (priv->individuals,
+      (gpointer) folks_individual_get_id (individual),
+      g_object_ref (individual));
+
+  g_signal_connect (individual, "group-changed",
+      G_CALLBACK (individual_group_changed_cb), self);
+  g_signal_connect (individual, "notify::is-favourite",
+      G_CALLBACK (individual_notify_is_favourite_cb), self);
+}
+
+static void
+remove_individual (EmpathyIndividualManager *self, FolksIndividual *individual)
+{
+  EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
+
+  g_signal_handlers_disconnect_by_func (individual,
+      individual_group_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (individual,
+      individual_notify_is_favourite_cb, self);
+
+  g_hash_table_remove (priv->individuals, folks_individual_get_id (individual));
+}
+
+/* This is emitted for *all* individuals in the individual aggregator (not
+ * just the ones we keep a reference to), to allow for the case where a new
+ * individual doesn't contain an EmpathyContact, but later has a persona added
+ * which does. */
+static void
+individual_notify_personas_cb (FolksIndividual *individual,
+    GParamSpec *pspec,
+    EmpathyIndividualManager *self)
+{
+  EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
+
+  const gchar *id = folks_individual_get_id (individual);
+  gboolean has_contact = empathy_folks_individual_contains_contact (individual);
+  gboolean had_contact = (g_hash_table_lookup (priv->individuals,
+      id) != NULL) ? TRUE : FALSE;
+
+  if (had_contact == TRUE && has_contact == FALSE)
+    {
+      GList *removed = NULL;
+
+      /* The Individual has lost its EmpathyContact */
+      removed = g_list_prepend (removed, individual);
+      g_signal_emit (self, signals[MEMBERS_CHANGED], 0, NULL, NULL, removed,
+          TP_CHANNEL_GROUP_CHANGE_REASON_NONE /* FIXME */);
+      g_list_free (removed);
+
+      remove_individual (self, individual);
+    }
+  else if (had_contact == FALSE && has_contact == TRUE)
+    {
+      GList *added = NULL;
+
+      /* The Individual has gained its first EmpathyContact */
+      add_individual (self, individual);
+
+      added = g_list_prepend (added, individual);
+      g_signal_emit (self, signals[MEMBERS_CHANGED], 0, NULL, added, NULL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_NONE /* FIXME */);
+      g_list_free (added);
+    }
+}
+
 static void
 aggregator_individuals_changed_cb (FolksIndividualAggregator *aggregator,
     GList *added,
@@ -100,28 +170,48 @@ aggregator_individuals_changed_cb (FolksIndividualAggregator *aggregator,
     guint reason,
     EmpathyIndividualManager *self)
 {
-  GList *l;
+  EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
+  GList *l, *added_filtered = NULL;
 
+  /* Filter the individuals for ones which contain EmpathyContacts */
   for (l = added; l; l = l->next)
     {
-      g_signal_connect (l->data, "group-changed",
-          G_CALLBACK (individual_group_changed_cb), self);
-      g_signal_connect (l->data, "notify::is-favourite",
-          G_CALLBACK (individual_notify_is_favourite_cb), self);
+      FolksIndividual *ind = FOLKS_INDIVIDUAL (l->data);
+
+      g_signal_connect (ind, "notify::personas",
+          G_CALLBACK (individual_notify_personas_cb), self);
+
+      if (empathy_folks_individual_contains_contact (ind) == TRUE)
+        {
+          add_individual (self, ind);
+          added_filtered = g_list_prepend (added_filtered, ind);
+        }
     }
 
   for (l = removed; l; l = l->next)
     {
-      g_signal_handlers_disconnect_by_func (l->data,
-          individual_group_changed_cb, self);
-      g_signal_handlers_disconnect_by_func (l->data,
-          individual_notify_is_favourite_cb, self);
+      FolksIndividual *ind = FOLKS_INDIVIDUAL (l->data);
+
+      g_signal_handlers_disconnect_by_func (ind,
+          individual_notify_personas_cb, self);
+
+      if (g_hash_table_lookup (priv->individuals,
+          folks_individual_get_id (ind)) != NULL)
+        remove_individual (self, ind);
     }
 
+  /* Bail if we have no individuals left */
+  if (added_filtered == NULL && removed == NULL)
+    return;
+
+  added_filtered = g_list_reverse (added_filtered);
+
   g_signal_emit (self, signals[MEMBERS_CHANGED], 0, message,
-      added, removed,
+      added_filtered, removed,
       tp_chanel_group_change_reason_from_folks_groups_change_reason (reason),
       TRUE);
+
+  g_list_free (added_filtered);
 }
 
 static void
@@ -129,6 +219,7 @@ individual_manager_dispose (GObject *object)
 {
   EmpathyIndividualManagerPriv *priv = GET_PRIV (object);
 
+  g_hash_table_destroy (priv->individuals);
   tp_clear_object (&priv->contact_manager);
   tp_clear_object (&priv->aggregator);
 
@@ -224,6 +315,8 @@ empathy_individual_manager_init (EmpathyIndividualManager *self)
 
   self->priv = priv;
   priv->contact_manager = empathy_contact_manager_dup_singleton ();
+  priv->individuals = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, g_object_unref);
 
   priv->aggregator = folks_individual_aggregator_new ();
   g_signal_connect (priv->aggregator, "individuals-changed",
@@ -241,12 +334,10 @@ GList *
 empathy_individual_manager_get_members (EmpathyIndividualManager *self)
 {
   EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
-  GHashTable *individuals;
 
   g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (self), NULL);
 
-  individuals = folks_individual_aggregator_get_individuals (priv->aggregator);
-  return individuals ? g_hash_table_get_values (individuals) : NULL;
+  return g_hash_table_get_values (priv->individuals);
 }
 
 FolksIndividual *
@@ -254,15 +345,10 @@ empathy_individual_manager_lookup_member (EmpathyIndividualManager *self,
     const gchar *id)
 {
   EmpathyIndividualManagerPriv *priv = GET_PRIV (self);
-  GHashTable *individuals;
 
   g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (self), NULL);
 
-  individuals = folks_individual_aggregator_get_individuals (priv->aggregator);
-  if (individuals != NULL)
-    return g_hash_table_lookup (individuals, id);
-
-  return NULL;
+  return g_hash_table_lookup (priv->individuals, id);
 }
 
 static void
@@ -401,7 +487,6 @@ empathy_individual_manager_remove_group (EmpathyIndividualManager *manager,
     const gchar *group)
 {
   EmpathyIndividualManagerPriv *priv;
-  GHashTable *individuals;
 
   g_return_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (manager));
   g_return_if_fail (group != NULL);
@@ -411,8 +496,7 @@ empathy_individual_manager_remove_group (EmpathyIndividualManager *manager,
   DEBUG ("removing group %s", group);
 
   /* Remove every individual from the group */
-  individuals = folks_individual_aggregator_get_individuals (priv->aggregator);
-  g_hash_table_foreach (individuals, (GHFunc) remove_group_cb,
+  g_hash_table_foreach (priv->individuals, (GHFunc) remove_group_cb,
       (gpointer) group);
 }
 
@@ -445,3 +529,71 @@ empathy_individual_manager_get_flags_for_connection (
 
   return flags;
 }
+
+static void
+link_personas_cb (FolksIndividualAggregator *aggregator,
+    GAsyncResult *async_result,
+    gpointer user_data)
+{
+  GError *error = NULL;
+
+  folks_individual_aggregator_link_personas_finish (aggregator, async_result,
+      &error);
+
+  if (error != NULL)
+    {
+      g_warning ("Failed to link personas: %s", error->message);
+      g_clear_error (&error);
+    }
+}
+
+void
+empathy_individual_manager_link_personas (EmpathyIndividualManager *self,
+    GList *personas)
+{
+  EmpathyIndividualManagerPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (self));
+  g_return_if_fail (personas != NULL);
+
+  priv = GET_PRIV (self);
+
+  DEBUG ("Linking %u personas", g_list_length (personas));
+
+  folks_individual_aggregator_link_personas (priv->aggregator, personas,
+      (GAsyncReadyCallback) link_personas_cb, NULL);
+}
+
+static void
+unlink_individual_cb (FolksIndividualAggregator *aggregator,
+    GAsyncResult *async_result,
+    gpointer user_data)
+{
+  GError *error = NULL;
+
+  folks_individual_aggregator_unlink_individual_finish (aggregator,
+      async_result, &error);
+
+  if (error != NULL)
+    {
+      g_warning ("Failed to unlink individual: %s", error->message);
+      g_clear_error (&error);
+    }
+}
+
+void
+empathy_individual_manager_unlink_individual (EmpathyIndividualManager *self,
+    FolksIndividual *individual)
+{
+  EmpathyIndividualManagerPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (self));
+  g_return_if_fail (FOLKS_IS_INDIVIDUAL (individual));
+
+  priv = GET_PRIV (self);
+
+  DEBUG ("Unlinking individual '%s'", folks_individual_get_id (individual));
+
+  folks_individual_aggregator_unlink_individual (priv->aggregator, individual,
+      (GAsyncReadyCallback) unlink_individual_cb, NULL);
+}
