@@ -91,8 +91,10 @@
 /* Name in the geometry file */
 #define GEOMETRY_NAME "main-window"
 
-/* Labels for empty contact list */
-#define NO_MATCH_FOUND _("No match found")
+enum {
+	PAGE_NO_MATCH = 0,
+	PAGE_CONTACT_LIST
+};
 
 G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
 
@@ -375,11 +377,16 @@ main_window_row_deleted_cb (GtkTreeModel      *model,
 
 		if (empathy_individual_view_is_searching (
 				priv->individual_view)) {
-			gtk_label_set_text (GTK_LABEL (priv->no_entry_label),
-					NO_MATCH_FOUND);
+			gchar *tmp;
+
+			tmp = g_strdup_printf ("<b><span size='xx-large'>%s</span></b>",
+				_("No match found"));
+
+			gtk_label_set_markup (GTK_LABEL (priv->no_entry_label), tmp);
+			g_free (tmp);
+
 			gtk_notebook_set_current_page (
-					GTK_NOTEBOOK (priv->notebook),
-					0);
+					GTK_NOTEBOOK (priv->notebook), PAGE_NO_MATCH);
 		}
 	}
 }
@@ -395,7 +402,7 @@ main_window_row_inserted_cb (GtkTreeModel      *model,
 	if (priv->empty) {
 		priv->empty = FALSE;
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
-				1);
+				PAGE_CONTACT_LIST);
 		gtk_widget_grab_focus (GTK_WIDGET (priv->individual_view));
 	}
 }
@@ -475,10 +482,19 @@ main_window_error_display (EmpathyMainWindow *window,
 	GtkWidget *action_table;
 	gchar     *str;
 	const gchar     *icon_name;
+	const gchar *error_message;
+	gboolean user_requested;
+
+	error_message =
+		empathy_account_get_error_message (account, &user_requested);
+
+	if (user_requested) {
+		return;
+	}
 
 	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
 					       tp_account_get_display_name (account),
-					       empathy_account_get_error_message (account));
+					       error_message);
 
 	info_bar = g_hash_table_lookup (priv->errors, account);
 	if (info_bar) {
@@ -1664,9 +1680,12 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			individual_manager);
 	g_object_unref (individual_manager);
 
+	/* For the moment, we disallow Persona drops onto the main contact list (e.g. from things such as
+	 * the EmpathyPersonaView in the linking dialogue). No code is hooked up to do anything on a Persona
+	 * drop, so allowing them would achieve nothing except confusion. */
 	priv->individual_view = empathy_individual_view_new (
 			priv->individual_store,
-			EMPATHY_INDIVIDUAL_VIEW_FEATURE_ALL,
+			EMPATHY_INDIVIDUAL_VIEW_FEATURE_ALL ^ EMPATHY_INDIVIDUAL_VIEW_FEATURE_PERSONA_DROP,
 			EMPATHY_INDIVIDUAL_FEATURE_ALL);
 
 	priv->butterfly_log_migration_members_changed_id = g_signal_connect (
