@@ -108,14 +108,64 @@ notification_close_helper (EmpathyStatusIconPriv *priv)
 }
 
 static void
-notification_action_cb (NotifyNotification *notification,
+notification_approve_cb (NotifyNotification *notification,
 			gchar              *action,
 			EmpathyStatusIcon  *icon)
 {
 	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
 
 	if (priv->event)
-		empathy_event_activate (priv->event);
+		empathy_event_approve (priv->event);
+}
+
+static void
+notification_decline_cb (NotifyNotification *notification,
+			gchar              *action,
+			EmpathyStatusIcon  *icon)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+
+	if (priv->event)
+		empathy_event_decline (priv->event);
+}
+
+static void
+add_notification_actions (EmpathyStatusIcon *self,
+			  NotifyNotification *notification)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (self);
+
+	switch (priv->event->type) {
+		case EMPATHY_EVENT_TYPE_CHAT:
+			notify_notification_add_action (notification,
+				"respond", _("Respond"), (NotifyActionCallback) notification_approve_cb,
+					self, NULL);
+			break;
+
+		case EMPATHY_EVENT_TYPE_VOIP:
+			notify_notification_add_action (notification,
+				"reject", _("Reject"), (NotifyActionCallback) notification_decline_cb,
+					self, NULL);
+
+			notify_notification_add_action (notification,
+				"answer", _("Answer"), (NotifyActionCallback) notification_approve_cb,
+					self, NULL);
+			break;
+
+		case EMPATHY_EVENT_TYPE_TRANSFER:
+		case EMPATHY_EVENT_TYPE_INVITATION:
+			notify_notification_add_action (notification,
+				"decline", _("Decline"), (NotifyActionCallback) notification_decline_cb,
+					self, NULL);
+
+			notify_notification_add_action (notification,
+				"accept", _("Accept"), (NotifyActionCallback) notification_approve_cb,
+					self, NULL);
+			break;
+
+		default:
+			break;
+	}
 }
 
 static void
@@ -172,15 +222,8 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 			}
 
 			if (empathy_notify_manager_has_capability (priv->notify_mgr,
-			           EMPATHY_NOTIFY_MANAGER_CAP_ACTIONS) &&
-			           priv->event->type != EMPATHY_EVENT_TYPE_PRESENCE) {
-				notify_notification_add_action (notification,
-					"respond",
-					_("Respond"),
-					(NotifyActionCallback) notification_action_cb,
-					icon,
-					NULL);
-			}
+			           EMPATHY_NOTIFY_MANAGER_CAP_ACTIONS))
+				add_notification_actions (icon, notification);
 
 			g_signal_connect (notification, "closed",
 					  G_CALLBACK (status_icon_notification_closed_cb), icon);
@@ -573,6 +616,7 @@ status_icon_finalize (GObject *object)
 	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->notify_mgr);
 	g_object_unref (priv->gsettings_ui);
+	g_object_unref (priv->window);
 }
 
 static void
@@ -681,13 +725,15 @@ empathy_status_icon_new (GtkWindow *window, gboolean hide_contact_list)
 			  G_CALLBACK (status_icon_delete_event_cb),
 			  icon);
 
-	should_hide = g_settings_get_boolean (priv->gsettings_ui,
+	if (!hide_contact_list) {
+		should_hide = g_settings_get_boolean (priv->gsettings_ui,
 			EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN);
-
-	if (gtk_window_is_active (priv->window) == should_hide) {
-		status_icon_set_visibility (icon, !should_hide, FALSE);
+	} else {
+		should_hide = TRUE;
 	}
 
+	status_icon_set_visibility (icon, !should_hide, FALSE);
+
 	return icon;
 }
 
