@@ -32,7 +32,7 @@
 #include <gdk/gdkx.h>
 #include <unique/unique.h>
 
-#if HAVE_LIBCHAMPLAIN
+#ifdef HAVE_LIBCHAMPLAIN
 #include <clutter-gtk/clutter-gtk.h>
 #endif
 
@@ -54,7 +54,6 @@
 #include <libempathy/empathy-connectivity.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-ft-factory.h>
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-tp-chat.h>
@@ -81,30 +80,6 @@ static void account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data);
 
-static void
-dispatch_cb (EmpathyDispatcher *dispatcher,
-    EmpathyDispatchOperation *operation,
-    gpointer user_data)
-{
-  GQuark channel_type;
-
-  channel_type = empathy_dispatch_operation_get_channel_type_id (operation);
-
-  if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
-    {
-      EmpathyFTFactory *factory;
-
-      factory = empathy_ft_factory_dup_singleton ();
-
-      /* if the operation is not incoming, don't claim it,
-       * as it might have been triggered by another client, and
-       * we are observing it.
-       */
-      if (empathy_dispatch_operation_is_incoming (operation))
-        empathy_ft_factory_claim_channel (factory, operation);
-    }
-}
-
 static void
 use_conn_notify_cb (GSettings *gsettings,
     const gchar *key,
@@ -228,6 +203,11 @@ unique_app_message_cb (UniqueApp *unique_app,
   return UNIQUE_RESPONSE_OK;
 }
 
+static gboolean show_version_cb (const char *option_name,
+    const char *value,
+    gpointer data,
+    GError **error) G_GNUC_NORETURN;
+
 static gboolean
 show_version_cb (const char *option_name,
     const char *value,
@@ -237,8 +217,6 @@ show_version_cb (const char *option_name,
   g_print ("%s\n", PACKAGE_STRING);
 
   exit (EXIT_SUCCESS);
-
-  return FALSE;
 }
 
 static void
@@ -310,41 +288,6 @@ account_manager_ready_cb (GObject *source_object,
   g_object_unref (gsettings);
 }
 
-static EmpathyDispatcher *
-setup_dispatcher (void)
-{
-  EmpathyDispatcher *d;
-  GPtrArray *filters;
-  struct {
-    const gchar *channeltype;
-    TpHandleType handletype;
-  } types[] = {
-    /* file transfer to contacts */
-    { TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, TP_HANDLE_TYPE_CONTACT  },
-  };
-  GHashTable *asv;
-  guint i;
-
-  /* Setup the an extended Client.Handler that matches everything we can do */
-  filters = g_ptr_array_new ();
-  for (i = 0 ; i < G_N_ELEMENTS (types); i++)
-    {
-      asv = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING, types[i].channeltype,
-        TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_INT, types[i].handletype,
-        NULL);
-
-      g_ptr_array_add (filters, asv);
-    }
-
-  d = empathy_dispatcher_new (PACKAGE_NAME"MoreThanMeetsTheEye", filters, NULL);
-
-  g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
-  g_ptr_array_free (filters, TRUE);
-
-  return d;
-}
-
 static void
 account_status_changed_cb (TpAccount *account,
     guint old_status,
@@ -354,14 +297,11 @@ account_status_changed_cb (TpAccount *account,
     GHashTable *details,
     EmpathyChatroom *room)
 {
-  TpConnection *conn;
-
-  conn = tp_account_get_connection (account);
-  if (conn == NULL)
+  if (new_status != TP_CONNECTION_STATUS_CONNECTED)
     return;
 
-  empathy_dispatcher_join_muc (conn,
-      empathy_chatroom_get_room (room), EMPATHY_DISPATCHER_NON_USER_ACTION);
+  empathy_dispatcher_join_muc (account,
+      empathy_chatroom_get_room (room), TP_USER_ACTION_TIME_NOT_USER_ACTION);
 }
 
 static void
@@ -408,9 +348,9 @@ account_manager_chatroom_ready_cb (GObject *source_object,
             }
           else
             {
-              empathy_dispatcher_join_muc (conn,
+              empathy_dispatcher_join_muc (account,
                   empathy_chatroom_get_room (room),
-                  EMPATHY_DISPATCHER_NON_USER_ACTION);
+                  TP_USER_ACTION_TIME_NOT_USER_ACTION);
             }
         }
 
@@ -445,7 +385,7 @@ empathy_idle_set_auto_away_cb (GSettings *gsettings,
 int
 main (int argc, char *argv[])
 {
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   EmpathyLocationManager *location_manager = NULL;
 #endif
   EmpathyStatusIcon *icon;
@@ -485,13 +425,15 @@ main (int argc, char *argv[])
 
   /* Init */
   g_thread_init (NULL);
+
+#ifdef HAVE_LIBCHAMPLAIN
+  gtk_clutter_init (&argc, &argv);
+#endif
+
   empathy_init ();
 
   optcontext = g_option_context_new (N_("- Empathy IM Client"));
   g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
-#if HAVE_LIBCHAMPLAIN
-  g_option_context_add_group (optcontext, clutter_get_option_group ());
-#endif
   g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
 
   if (!g_option_context_parse (optcontext, &argc, &argv, &error)) {
@@ -552,9 +494,9 @@ main (int argc, char *argv[])
   tp_account_manager_prepare_async (account_manager, NULL,
       account_manager_ready_cb, NULL);
 
-  /* Handle channels */
-  dispatcher = setup_dispatcher ();
-  g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
+  /* The EmpathyDispatcher doesn't dispatch anything any more but we have to
+   * keep it around as we still use it to request channels */
+  dispatcher = empathy_dispatcher_dup_singleton ();
 
   migrate_config_to_xdg_dir ();
 
@@ -592,8 +534,14 @@ main (int argc, char *argv[])
   g_signal_connect (ft_factory, "new-incoming-transfer",
       G_CALLBACK (new_incoming_transfer_cb), NULL);
 
+  if (!empathy_ft_factory_register (ft_factory, &error))
+    {
+      g_warning ("Failed to register FileTransfer handler: %s", error->message);
+      g_error_free (error);
+    }
+
   /* Location mananger */
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   location_manager = empathy_location_manager_dup_singleton ();
 #endif
 
@@ -613,7 +561,7 @@ main (int argc, char *argv[])
   g_object_unref (log_manager);
   g_object_unref (dispatcher);
   g_object_unref (chatroom_manager);
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   g_object_unref (location_manager);
 #endif
   g_object_unref (ft_factory);
